Program TransparencyMap;
{ Creates a transparency lookup table }
{$G+,X+}

Uses Crt;

Type
  tMap = Array[0..256*256-2] of Byte;
  pMap = ^tMap;
  RGBType = Record
    r, g, b : Byte;
  End;
  PaletteType = Array[0..255] of RGBType;

Var
  TransparencyTable : pMap;
  Palette : PaletteType;
  View : Boolean;
  InFile, OutFile : String;
  SqrTable : Array[0..255] of Word;

Procedure CalcSqrTable;

Var
  i : Integer;

Begin
  For i := 0 to 255 do
    SqrTable[i] := Sqr(i);
End;

Procedure LoadPalette(Filename : String);

Var
  Diskfile : File;

Begin
  Assign(Diskfile, Filename);
  Reset(Diskfile,1);
  BlockRead(Diskfile, Palette, 768);
  Close(Diskfile);
End;

Function MatchColor(R, G, B : Integer; Var Pal : PaletteType) : Integer;

Var
  Error, Temp : Word;
  i, Match : Byte;
  p : ^RGBType;

Begin
  p := @Pal;
  For i := 0 to 255 do
    Begin
      Temp := SqrTable[Abs(R-p^.r)] +
              SqrTable[Abs(G-P^.g)] +
              SqrTable[Abs(B-P^.b)];
      If Temp < Error
        Then Begin
          Error := Temp;
          Match := i;
        End;
      Inc(Longint(P), 3);
    End;
  MatchColor := Match;
End;

Procedure CalcTable;

Var
  FrontColor : Integer;
  BackColor  : Integer;
  R1, G1, B1, R2, G2, B2 : Byte;
  NewR, NewG, NewB : Byte;

Begin
  Writeln('Making Transparency Table');
  Write('[', ']':65);
  GotoXY(2, WhereY);
  For FrontColor := 0 to 255 do
    Begin
      If FrontColor And 3 = 0
        Then Write('.');
      R1 := Palette[FrontColor].r;
      G1 := Palette[FrontColor].g;
      B1 := Palette[FrontColor].b;
      For BackColor := FrontColor to 255 do
        Begin
          R2 := Palette[BackColor].r;
          G2 := Palette[BackColor].g;
          B2 := Palette[BackColor].b;

          NewR := (R1+R2) Shr 1;
          NewG := (G1+G2) Shr 1;
          NewB := (B1+B2) Shr 1;

          TransparencyTable^[FrontColor*256+BackColor] := MatchColor(NewR, NewG, NewB, Palette);
        End;
    End;
  For FrontColor := 0 to 255 do
    For BackColor := FrontColor to 255 do
      TransparencyTable^[BackColor*256+FrontColor] := TransparencyTable^[FrontColor*256+BackColor];
  Writeln;
End;

Procedure SaveTransParencyTable(Filename : String);

Var
  Diskfile : File;
  Count : Word;

Begin
  Assign(Diskfile, Filename);
  ReWrite(Diskfile, 1);
  For Count := 0 to 255 do
    BlockWrite(Diskfile, TransparencyTable^[Count*256], 256);
  Close(Diskfile);
End;

Procedure SetMode(M : Integer); Assembler;

Asm
  Mov  ax,M
  Int  10h
End;

Procedure DisplayTransparencyTable;

Var
  i : Integer;

Begin
  SetMode($13);
  For i := 0 to 255 do
    Begin
      Port[$3c8] := i;
      Port[$3c9] := Palette[i].r;
      Port[$3c9] := Palette[i].g;
      Port[$3c9] := Palette[i].b;
    End;
  For i := 0 to 199 do
    Move(TransparencyTable^[i*256], Mem[$A000:i*320], 256);
  ReadKey;
  SetMode($03);
End;

Function UpStr(S : String) : String;

Var
  Temp : String;
  C : Integer;

Begin
  Temp := '';
  For C := 1 to Length(S) do
    Temp := Temp + UpCase(S[C]);
  UpStr := Temp;
End;


Procedure Help;

Begin
  Writeln('Usage: TRANS {file} {options}');
  Writeln('Options: (preceded by ''/'' or ''-'')');
  Writeln;
  Writeln('                 [File Specifiers]');
  Writeln('{file} specifies <infile> [outfile]');
  Writeln('   <infile>: input palette file (.PAL is default extention)');
  Writeln('   <outfile>: optional transparency table name (.TT is default extention)');
  Writeln;
  Writeln('                 [Other Options]');
  Writeln(' /V  view the palette table after its created.');
  Writeln;
  Writeln('                 [Help]');
  Writeln(' [/h | /?]  displays this help.');
  Halt(0);
End;

Procedure ParseCommandLine;

Var
  i : Integer;
  Temp : String;
  InFlag : Boolean;
  OutFlag : Boolean;

Begin
  Writeln('TRANS - Transparency Table v0.90                     Alex Chalfin 10/8/95');
  Writeln;
  View := False;
  InFlag := False;
  OutFlag := False;
  If ParamCount = 0
    Then Help;
  For i := 1 to ParamCount do
    Begin
      Temp := UpStr(ParamStr(i));
      If (Temp[1] = '-') or (Temp[1] = '/')
        Then Begin
          If Temp[2] = 'V'
            Then View := True;
          If Temp[2] = 'H'
            Then Help;
          If Temp[2] = '?'
            Then Help;
        End
        Else Begin
          If Not(InFlag)
            Then Begin
              InFile := Temp;
              InFlag := True;
            End
            Else Begin
              OutFile := Temp;
              OutFlag := True;
            End;
        End;
    End;
  If Not(InFlag)
    Then Begin
      Writeln('No input file specified!');
      Halt(0);
    End;
  If Pos('.', InFile) = 0
    Then InFile := InFile + '.PAL';
  If Not(OutFlag)
    Then Begin
      OutFile := InFile;
      While(OutFile[Length(OutFile)] <> '.') do
        Delete(OutFile, Length(OutFile), 1);
      OutFile := OutFile + 'TT';
    End
    Else Begin
      If Pos('.', OutFile) = 0
        Then OutFile := OutFile + '.TT';
    End;
End;

Begin
  ParseCommandLine;
  CalcSqrTable;
  LoadPalette(InFile);
  New(TransparencyTable);
  FillChar(TransparencyTable^, 65535, 0);
  CalcTable;
  SaveTransParencyTable(OutFile);
  If View
    Then DisplayTransparencyTable;
  Dispose(TransparencyTable);
End.